# -*- coding: utf-8 -*-
#
# Created on Nov 27, 2015
# @authors: Jiri Spilka
# http://people.ciirc.cvut.cz/~spilkjir
# @2015, CIIRC, Czech Technical University in Prague
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see CTGViewerLite.py for details)

"""
Annotator
---------
Some text

Reference
~~~~~~~~~

.. autoclass:: Annotator
    :members:
    :private-members:

"""

from PyQt4 import QtGui, Qt
from PyQt4.QtCore import pyqtSignal
import PyQt4.Qwt5 as Qwt
import numpy as np
import itertools
import os
import textwrap
import logging

import Common
from GuiForms import AddNote
from Config import ConfigStatic


class EnumAnnType:
    """
    Types of annotations: basal, baseline, recovery
    """
    def __init__(self):
        pass

    select = 'select'
    basal = 'basal'
    baseline = 'baseline'
    recovery = 'recovery'
    note = 'note'


class Annotator:
    """
    Class Annotator manages loading and saving of annotations stored in an .ann file.
    """

    def __init__(self):

        self._ann_extension = '.ann'
        self._ann_file = ''
        self._path = ''
        self._dannotations = dict()

        self._sep_items = ','
        self._sep_anns = '\n'

        self._log = logging.getLogger(ConfigStatic.logger_name)

    def __load_annotations(self):
        """
        Load annotations from .ann file.

        :return: annotations in dict()
        :rtype: dict()
        """

        self._dannotations = dict()

        if self.get_signal_annotated():
            self._log.info("Loading annotation file: {0}".format(self._ann_file))
            with open(self._ann_file, 'r') as fp:
                for line in fp.readlines():
                    # print line.split(',')

                    s = line.split(',')

                    if len(s) != 5:
                        continue

                    curve_type = str(s[1])
                    x_from = int(s[2])
                    x_to = int(s[3])

                    if not(curve_type == EnumAnnType.basal or curve_type == EnumAnnType.baseline or
                            curve_type == EnumAnnType.recovery or curve_type == EnumAnnType.note):
                        raise Exception('Unsuported annotation type')

                    if curve_type == EnumAnnType.note:
                        note = PyQwtPlotMarkerAnnotator(curve_type, x_from, x_to, None, str(s[4]))
                        note.id = int(s[0])
                        self._dannotations[note.id] = note
                    else:
                        curve = PyQwtPlotCurveAnnotator(curve_type, x_from, x_to, int(s[4]))
                        curve.id = int(s[0])
                        self._dannotations[curve.id] = curve
        else:
            self._dannotations = dict()

        return self._dannotations

    def ann_file_load(self, data_file):
        """
        Set the annotation file and load it.

        :param data_file: annotation file
        :type data_file: str
        :return:
        """
        name = Common.get_filename_without_ext(data_file)

        self._path = os.path.dirname(data_file)
        # set annotations
        self._ann_file = os.path.join(self._path, name + self._ann_extension)

        # load annotations
        self.__load_annotations()
        return self._dannotations

    def get_signal_annotated(self):
        b = Common.file_exists(self._ann_file) and Common.get_mumber_lines(self._ann_file) > 0
        return b

    def get_annotations(self):
        return self._dannotations

    def get_annotations_copy(self):

        ann = dict()
        for key, curve in self._dannotations.iteritems():
            typ = curve.get_curve_type()
            if typ == EnumAnnType.note:
                c = PyQwtPlotMarkerAnnotator(typ, curve.x_from, curve.x_to, curve.yval, str(curve.text.text()))
            else:
                c = PyQwtPlotCurveAnnotator(typ, curve.x_from, curve.x_to, curve.yval)

            ann[curve.id] = c

        return ann

    def set_annotations(self, dann):
        self._dannotations = dann

    def save_annotations(self):

        with open(self._ann_file, 'w+') as fp:
            for idd, d in self._dannotations.items():

                curve_type = str(d.get_curve_type())

                if d.valid:
                    fp.write(str(idd))
                    fp.write(self._sep_items)
                    fp.write(curve_type)
                    fp.write(self._sep_items)
                    fp.write(str(d.x_from))
                    fp.write(self._sep_items)
                    fp.write(str(d.x_to))
                    fp.write(self._sep_items)

                    if curve_type == EnumAnnType.note:
                        s = str(d.text.text())
                        print s
                        s = s.replace('\n', '')
                        print s
                        fp.write(s)
                    else:
                        fp.write(str(d.yval))

                    fp.write(self._sep_anns)

        self._log.info("Saving annotation file: {0}".format(self._ann_file))

    def set_annotations_and_save(self, dann):
        self.set_annotations(dann)
        self.save_annotations()


class PyQwtPlotCurveAnnotator(Qwt.QwtPlotCurve):
    """
    Definitions for basal heart rate, baseline, and recovery
    """
    newid = itertools.count().next

    def __init__(self, curve_type=None, x1=0, x2=0, y=0):

        self.id = PyQwtPlotCurveAnnotator.newid()
        Qwt.QwtPlotCurve.__init__(self)

        # information to be saved in annotation file
        self.__curve_type = curve_type
        self.x_from = x1
        self.x_to = x2
        self.yval = y
        self.valid = True  # if a curve is valid and should be plotted

        #  QPen settings
        self._pen = Qt.QPen()
        self._pen_symbol = Qt.QPen()
        self._symbol = Qwt.QwtSymbol()

        self._color_highlight = Qt.Qt.darkMagenta

        if curve_type == EnumAnnType.basal:
            c = Qt.QColor(Qt.Qt.gray)
            c.setAlpha(0)
            self.set_pen(Qt.Qt.gray)
            self.set_pen_symbol(c)
            self.set_symbol()

        elif curve_type == EnumAnnType.baseline:
            self.set_pen(Qt.Qt.blue, 4, Qt.Qt.DotLine)
            self.set_pen_symbol(Qt.Qt.blue)
            self.set_symbol(Qwt.QwtSymbol.VLine, Qt.Qt.blue)

        elif curve_type == EnumAnnType.recovery:
            self.set_pen(Qt.Qt.green)
            self.set_pen_symbol(Qt.Qt.green)
            self.set_symbol(Qwt.QwtSymbol.VLine, Qt.Qt.green, Qt.QSize(15, 15))

        elif curve_type == EnumAnnType.note:
            c = Qt.QColor(Qt.Qt.gray)
            c.setAlpha(0)
            self.set_pen(c)
            self.set_pen_symbol(c)
            self.set_symbol()

        else:
            raise Exception('Given type of curve does not exist')

        self._default_pen = Qt.QPen(self._pen)

    def get_default_pen(self):
        return self._default_pen

    def get_symbol(self):
        return self._symbol

    def get_symbol_highlight(self):
        symbol = Qwt.QwtSymbol(self._symbol)
        pen = Qt.QPen(self._pen_symbol)
        pen.setColor(self._color_highlight)
        symbol.setPen(pen)
        return symbol

    def get_color_highlight(self):
        return self._color_highlight

    def get_curve_type(self):
        return self.__curve_type

    def set_pen(self, c=Qt.Qt.gray, w=4, l=Qt.Qt.SolidLine):
        self._pen = Qt.QPen(c)
        self._pen.setWidth(w)
        self._pen.setStyle(l)
        self.setPen(self._pen)

    def set_pen_symbol(self, c=Qt.Qt.black, w=4, l=Qt.Qt.SolidLine):
        self._pen_symbol = Qt.QPen(c)
        self._pen_symbol.setWidth(w)
        self._pen_symbol.setStyle(l)

    def set_symbol(self, s=Qwt.QwtSymbol.VLine, c=Qt.Qt.black, qsize=Qt.QSize(15, 15)):
        self._symbol = Qwt.QwtSymbol(s, Qt.QBrush(c), self._pen_symbol, qsize)
        self.setSymbol(self._symbol)

    def set_pen_symbol_light(self):

        self._pen.setWidth(3)
        self.setPen(self._pen)

        c = Qt.QColor(Qt.Qt.gray)
        c.setAlpha(0)
        self._pen_symbol.setColor(c)
        self.set_symbol(Qwt.QwtSymbol.VLine, c, Qt.QSize(4, 4))

    def set_xy_values(self, x1, x2, y):

        self.x_from = x1
        self.x_to = x2
        self.yval = y

    def correct_x_val_order(self):

        if self.x_from > self.x_to:
            self.x_from, self.x_to = self.x_to, self.x_from

    def plot(self):

        x = [self.x_from, self.x_to]
        y = self.yval * np.ones((len(x), 1), float)

        self.setData(x, y)

    def plot_xy(self, x1, x2, y):

        x = [x1, x2]
        y = y * np.ones((len(x), 1), float)

        self.setData(x, y)

    def draw_my(self, x, y):
        self.draw(x, y)


class PyQwtPlotMarkerAnnotator(Qwt.QwtPlotMarker):
    """
    Definition of annotation box
    """
    newid = itertools.count().next

    def __init__(self, curve_type=None, x1=None, x2=None, y=None, s=""):

        self.id = 10000+PyQwtPlotMarkerAnnotator.newid()
        Qwt.QwtPlotMarker.__init__(self)

        # information to be saved in annotation file
        self.__curve_type = curve_type
        self.x_from = x1
        self.x_to = x2
        self.yval = y
        self.valid = True  # if a curve is valid and should be plotted
        self.text = Qwt.QwtText(self.text_wrap(s))

        self.font_app = QtGui.QFont()
        #  QPen settings
        self._pen = Qt.QPen()
        self._pen_symbol = Qt.QPen()
        self._symbol = Qwt.QwtSymbol()

        self._color_highlight = Qt.Qt.darkMagenta

        if curve_type == EnumAnnType.note:

            self.setLineStyle(Qwt.QwtPlotMarker.VLine)
            self.setLabelAlignment(Qt.Qt.AlignRight | Qt.Qt.AlignTop)

            c = Qt.QColor(Qt.Qt.black)
            c.setAlpha(30)
            self.set_pen(c, 4)
            self.setLinePen(self._pen)

            self._set_text_property()

            c = Qt.QColor(Qt.Qt.gray)
            c.setAlpha(0)
            self.set_pen_symbol(c)
            self.set_symbol()

        else:
            raise Exception('Given type of curve does not exist')

        self._default_pen = Qt.QPen(self._pen)

    def get_pen(self):
        return self._pen

    def get_default_pen(self):
        return self._default_pen

    def get_symbol(self):
        return self._symbol

    def get_symbol_highlight(self):
        symbol = Qwt.QwtSymbol(self._symbol)
        pen = Qt.QPen(self._pen_symbol)
        pen.setColor(self._color_highlight)
        symbol.setPen(pen)
        return symbol

    def get_color_highlight(self):
        return self._color_highlight

    def get_curve_type(self):
        return self.__curve_type

    def text_wrap(self, s):
        w = 140
        s = textwrap.fill(s, w/8+1)
        return s

    def set_pen(self, c=Qt.Qt.gray, w=4, l=Qt.Qt.SolidLine):
        self._pen = Qt.QPen(c)
        self._pen.setWidth(w)
        self._pen.setStyle(l)
        self.setPen(self._pen)

    def set_pen_symbol(self, c=Qt.Qt.black, w=4, l=Qt.Qt.SolidLine):
        self._pen_symbol = Qt.QPen(c)
        self._pen_symbol.setWidth(w)
        self._pen_symbol.setStyle(l)

    def set_symbol(self, s=Qwt.QwtSymbol.VLine, c=Qt.Qt.black, qsize=Qt.QSize(15, 15)):
        self._symbol = Qwt.QwtSymbol(s, Qt.QBrush(c), self._pen_symbol, qsize)
        self.setSymbol(self._symbol)

    def set_marker(self):
        pass

    def _set_text_property(self):
        c = Qt.QColor(Qt.Qt.black)
        self.text.setColor(c)
        c.setAlpha(20)
        self.text.setBackgroundBrush(c)
        self.font_app.setBold(False)
        self.text.setFont(self.font_app)

    def set_default_pen(self):
        self._default_pen = Qt.QPen(self._pen)

    def set_pen_symbol_light(self):

        self.setLabelAlignment(Qt.Qt.AlignRight | Qt.Qt.AlignBottom)
        font = Qt.QFont(self.font_app)
        # print self.font_app.pointSize()
        font.setPointSize(font.pointSize()-2)
        # print font.pointSize()
        self.text.setFont(font)

    def set_xy_values(self, x1, x2=0, y=0):
        self.set_x(x1)

    def set_x(self, x1):
        self.x_from = x1
        self.x_to = x1

    def set_y(self, y):
        self.text.setText(y)

    def correct_x_val_order(self):
        """not used for PyQwtPlotMarkerAnnotator"""
        pass

    def plot(self):
        self.setLabel(self.text)
        self.setValue(self.x_from, 0)

    def plot_xy(self, x1, x2=0, y=0):
        self.setValue(x1, 0)

    def setPen(self, pen):
        self.setLinePen(pen)

    def draw_my(self, x, y):
        """auxially function"""
        self.plot()


# class AnnotatorTemp(Qwt.QwtPlot):
#     """
#
#     """
#
#     def __init__(self, *args):
#         Qwt.QwtPlot.__init__(self, *args)
#
#         self.setCanvasBackground(Qt.Qt.white)
#
#         grid = Qwt.QwtPlotGrid()
#         grid.attach(self)
#         grid.setMajPen(Qt.QPen(Qt.Qt.black, 0, Qt.Qt.DotLine))
#
#         self.setAxisScale(Qwt.QwtPlot.xBottom, 0.0, 1000.0)
#         self.setAxisScale(Qwt.QwtPlot.yLeft, 0.0, 50.0)
#
#         self.canvas_picker = CanvasPickerAnnotator(self)
#         # self.canvas_picker.signal_point_clicked.connect(self._point_selected)
#
#         self.d_ann_all_curves = dict()
#
#         curve_baseline = PyQwtPlotCurveAnnotator(EnumAnnType.baseline)
#         curve_recovery = PyQwtPlotCurveAnnotator(EnumAnnType.recovery)
#
#         curve_baseline.set_xy_values(50, 700, 40)
#         curve_recovery.set_xy_values(300, 400, 10)
#
#         self.d_ann_all_curves[curve_baseline.id] = curve_baseline
#         self.d_ann_all_curves[curve_recovery.id] = curve_recovery
#
#         self._ann_plot_curves()
#
#         self.ann_basal()
#
#         self.__action_draw = 0
#
#         for curve in self.itemList():
#             if isinstance(curve, Qwt.QwtPlotCurve):
#                 print curve
#
#     def ann_basal(self):
#         curve_basal = PyQwtPlotCurveAnnotator(EnumAnnType.basal)
#         curve_basal.set_xy_values(0, 1000, 30)
#         self.d_ann_all_curves[curve_basal.id] = curve_basal
#         self._ann_plot_curves()
#
#     def ann_baseline_start(self, pos):
#         curve_baseline = PyQwtPlotCurveAnnotator(EnumAnnType.baseline)
#         curve_baseline.set_xy_values(pos.x(), pos.x(), pos.y())
#         self.d_ann_all_curves[curve_baseline.id] = curve_baseline
#         self._ann_plot_curves()
#
#     def _ann_plot_curves(self):
#
#         for key, value in self.d_ann_all_curves.iteritems():
#             value.attach(self)
#             value.plot()
#
#         self.replot()


class CanvasPickerAnnotator(Qt.QObject):
    """
    Pick events generated by user such as draw basal, baseline, recovery, etc.
    The CanvasPickerAnnotator calls super function to plot the events

    :param plot: x axis values
    :type plot: FhrPlot
    """

    signal_ann_moved = pyqtSignal()
    signal_ann_moved_outside_view = pyqtSignal(['int'])

    def __init__(self, p_plot):
        Qt.QObject.__init__(self, p_plot)
        self.__selected_curve = None
        self.__selected_point = -1
        self.__point_clicked = None

        # self.__annotator = annotator
        # self.__plot = annotator.pyqwt_plot
        self.__plot = p_plot

        self._add_note = AddNote()
        # self._add_note.setWindowFlags(Qt.Qt.WindowStaysOnTopHint)
        # self._add_note.show()
        # self._fhrDraw.attach(self)

        self.canvas = self.__plot.canvas()
        self.canvas.installEventFilter(self)

        # # We want the focus, but no focus rect.
        # # The selected point will be highlighted instead.
        self.canvas.setFocusPolicy(Qt.Qt.StrongFocus)
        # canvas.setCursor(Qt.Qt.PointingHandCursor)
        self.canvas.setFocusIndicator(Qwt.QwtPlotCanvas.ItemFocusIndicator)
        self.canvas.setFocus()

        self._distance_point = 100
        self._distance_line = 5
        self._distance_note = 50

        self.__bleft_btn_hold = False
        self.__bmoving = False
        # self.__ann_action = self.__plot.get_ann_action()

        # self.canvas.setMouseTracking(True)
        # self.__shiftCurveCursor(True)

    def __select(self, pos, bhighlight=True, select_type=None):
        """

        :param pos:
        :param bhighlight:
        :param select_type:
        :type pos: QPointF
        :return:
        """
        # found, distance, point = None, 100, -1

        dmax_line = self._distance_line
        dmax_point = self._distance_point
        dmax_note = self._distance_note
        found_curve = None
        found_point = None

        point_dist = np.infty
        line_dist = np.infty
        note_dist = np.infty
        point_i = -1

        self.__unselect()

        for key, curve in self.__plot.d_ann_all_curves.iteritems():

            t = curve.get_curve_type()

            if select_type is not None:
                if t != select_type:
                    continue

            if t == EnumAnnType.basal:
                line_dist = abs(pos.y() - curve.y(0))

            elif t == EnumAnnType.baseline or t == EnumAnnType.recovery:
                d0 = abs(curve.x_from - pos.x())
                d1 = abs(curve.x_to - pos.x())
                # print d0, d1

                if d0 < d1:
                    point_i = 0
                    point_dist = d0
                else:
                    point_i = 1
                    point_dist = d1

                line_dist = abs(pos.y() - curve.y(0))

            elif t == EnumAnnType.note:
                note_dist = abs(pos.x() - curve.x_from)
                # print note_dist

            else:
                return 0

            if point_dist < dmax_point and line_dist < dmax_line:
                # if a point is close
                found_curve = curve
                found_point = point_i
                dmax_point = point_dist
                dmax_line = line_dist

            elif line_dist <= dmax_line and curve.x_from < pos.x() < curve.x_to:
                # if a line is close
                found_curve = curve
                found_point = -1
                dmax_line = line_dist

            elif note_dist < dmax_note:
                # if a note is close
                found_curve = curve
                found_point = -1
                dmax_note = note_dist

        if found_curve is not None:
            self.__selected_curve = found_curve
            self.__selected_point = found_point
            self.__highlight(bhighlight)
            self.__point_clicked = pos
            return True

        return False

    def __select_curve(self, curve, point, pos):
        self.__selected_curve = curve
        self.__selected_point = point
        self.__point_clicked = pos

    def __unselect(self):
        self.__highlight(False)
        self.__selected_curve = None
        self.__selected_point = -1

    # def __highlight_old(self, b_highlight):
    #
    #     curve = self.__selected_curve
    #     bdrawpoint = self.__selected_point != -1
    #     if not curve:
    #         return
    #
    #     # Use copy constructors to defeat the reference semantics.
    #     symbol = curve.get_symbol()
    #     new_symbol = curve.get_symbol()
    #
    #     newpen = Qt.QPen(curve.get_default_pen())
    #
    #     if b_highlight:
    #         if bdrawpoint:
    #             new_symbol = curve.get_symbol_highlight()
    #         else:
    #             newpen.setColor(curve.get_color_highlight())
    #
    #     doreplot = self.__plot.autoReplot()
    #
    #     self.__plot.setAutoReplot(False)
    #
    #     if bdrawpoint:
    #         curve.setSymbol(new_symbol)
    #
    #     curve.draw(self.__selected_point, self.__selected_point)
    #     # else:
    #     if bdrawpoint:
    #         curve.setSymbol(symbol)
    #     else:
    #         curve.setPen(newpen)
    #         curve.draw(curve.xAxis(), curve.yAxis())
    #
    #     self.__plot.setAutoReplot(doreplot)
    #     # self.__plot.replot()

    def __highlight(self, b_highlight):

        curve = self.__selected_curve
        bdrawpoint = self.__selected_point != -1
        if not curve:
            return

        # Use copy constructors to defeat the reference semantics.
        # symbol = curve.get_symbol()
        new_symbol = curve.get_symbol()
        newpen = Qt.QPen(curve.get_default_pen())

        if b_highlight:
            new_symbol = curve.get_symbol_highlight()
            newpen.setColor(curve.get_color_highlight())

        doreplot = self.__plot.autoReplot()
        self.__plot.setAutoReplot(False)

        if curve.get_curve_type() != EnumAnnType.basal:
            curve.setSymbol(new_symbol)

        # curve.draw(self.__selected_point, self.__selected_point)
        # else:
        # if bdrawpoint:
        #     curve.setSymbol(symbol)
        # else:
        curve.setPen(newpen)
        # print curve.xAxis(), curve.yAxis()
        curve.draw_my(curve.xAxis(), curve.yAxis())

        self.__plot.setAutoReplot(doreplot)
        self.__plot.replot()

    def __delete(self):

        curve = self.__selected_curve
        if not curve:
            return

        self.__plot.ann_delete_curve(curve)

    def __move(self, pos, bfinished_move=False):
        """
        Move selected item

        :param pos: x and y position
        :param bfinished_move: if MouseRelease then moving was finished
        :type  pos: QPointF
        :type  bfinished_move: bool
        """

        curve = self.__selected_curve
        bdragpoint = self.__selected_point != -1

        if not curve:
            return

        # if isinstance(curve, PyQwtPlotMarkerAnnotator):
        #     pos = self._compute_point(pos)

        # x = self.__plot.invTransform(curve.xAxis(), pos.x())
        # y = self.__plot.invTransform(curve.yAxis(), pos.y())
        x = pos.x()
        y = pos.y()

        # if x < self.__plot.xAxisMinSample:
        #     x = self.__xAxisMinSample

        xmin = self.__plot.viewXMinSample()
        xmax = self.__plot.viewXMaxSample()

        # if outside of box then move plot
        if x > xmax:
            self.signal_ann_moved_outside_view.emit(x-xmax)
        elif x < xmin:
            self.signal_ann_moved_outside_view.emit(x-xmin)

        # if a point is dragged
        if bdragpoint:
            if self.__selected_point == 0:
                curve.set_xy_values(int(x), curve.x_to, curve.yval)
                curve.plot()
            else:
                curve.set_xy_values(curve.x_from, int(x), curve.yval)
                curve.plot()

            if bfinished_move:
                curve.correct_x_val_order()
                # print curve.x_from
                # print curve.x_to
                self.signal_ann_moved.emit()
        else:
            d = int(x - self.__point_clicked.x())

            if curve.get_curve_type() != EnumAnnType.basal:
                x_new_from = curve.x_from + d
                x_new_to = curve.x_to + d
            else:
                x_new_from = curve.x_from
                x_new_to = curve.x_to

            x_new_from, x_new_to, y = self.check_outside_of_plot(x_new_from, x_new_to, y)
            curve.plot_xy(x_new_from, x_new_to, y)

            if bfinished_move:
                curve.set_xy_values(int(x_new_from), int(x_new_to), y)
                self.signal_ann_moved.emit()

        self.__plot.replot()

    def check_outside_of_plot(self, x1, x2, y):

        xmin = self.__plot.xAxisMinSample()
        xmax = self.__plot.xAxisMaxSample()

        ymin = self.__plot.viewYMinSample()
        ymax = self.__plot.viewYMaxSample()

        x1 = x1 if x1 > xmin else xmin
        x1 = x1 if x1 < xmax else xmax

        x2 = x2 if x2 > xmin else xmin
        x2 = x2 if x2 < xmax else xmax

        y = y if y > ymin else ymin
        y = y if y < ymax else ymax

        return x1, x2, y

    # def event(self, event):
    #     if event.type() == Qt.QEvent.User:
    #         return True
    #     return Qt.QObject.event(event)

    def eventFilter(self, object, event):

        # print 'event'
        # print event.type()
        action = self.__plot.get_ann_action()

        if event.type() == Qt.QEvent.Paint:
            Qt.QApplication.postEvent(self, Qt.QEvent(Qt.QEvent.User))

        elif event.type() == Qt.QEvent.MouseButtonPress:  # and self.__bleft_btn_hold is False:

            point = self._compute_point(event.pos())

            if event.button() == Qt.Qt.LeftButton:

                # print object
                # print object.metaObject().className()
                # print event

                self.__unselect()
                # print self.__plot.get_ann_action()

                # bstart_draw = False
                if action == EnumAnnType.basal:
                    self.__plot.ann_basal(point.y())
                    self.__bleft_btn_hold = False

                elif action == EnumAnnType.baseline or action == EnumAnnType.recovery:
                    curve = self.__plot.ann_line_start(point, action)
                    self.__select_curve(curve, point, point)
                    self.__bleft_btn_hold = True
                    # bstart_draw = True

                elif action == EnumAnnType.note:
                    self.__bleft_btn_hold = False
                    self._add_note.clear_text()
                    if self.__select(point, True, EnumAnnType.note):
                        # modify current annotation
                        self._add_note.set_text(self.__selected_curve.text.text())
                        point = Qt.QPointF(self.__selected_curve.x_from, 0)

                    # delete current and add new annotation
                    if self._add_note.show():
                        self.__delete()
                        self.__plot.ann_note(point, self._add_note.get_text())
                    else:
                        return True

                elif action == EnumAnnType.select:
                    self.__select(point)

                # # Select line/point
                # # The selection is conveniently placed after insertion of new item.
                # # When action new item is inserted the select is called and the item is drawed.
                # if self.__plot.get_ann_action() == EnumAnnType.select:
                #     self.__select(point)
                # elif bstart_draw:
                #     self.__select(point, False)

            # elif event.button() == Qt.Qt.RightButton:
            #     if self.__plot.get_ann_action() == EnumAnnType.note:
            #         if self.__select(point, True, EnumAnnType.note):
            #             pass

        elif event.type() == Qt.QEvent.MouseMove:
            # print 'MouseMove'
            point = self._compute_point(event.pos())
            self.__bmoving = True
            self.__move(point)
            return True

        elif event.type() == Qt.QEvent.MouseButtonRelease:

            if self.__bmoving:
                point = self._compute_point(event.pos())
                self.__bmoving = False
                self.__move(point, True)
                self.__bleft_btn_hold = False
            else:
                # only single click at plot. Recovery and baseline are to short -> delete
                self.__bleft_btn_hold = False
                if action == EnumAnnType.baseline or action == EnumAnnType.recovery:
                    self.__delete()

        elif event.type() == Qt.QEvent.KeyPress:
            if event.key() == Qt.Qt.Key_Delete:
                self.__delete()

        return Qwt.QwtPlot.eventFilter(self, object, event)

    def _compute_point(self, pos):
        """
        The clicked point stored in event.x() and event.y() is transformed to x and y axis for
        particular plot attached.

        :param pos: x and y position
        :type  pos: QPointF
        :rtype: QPointF
        """
        clickedpointx = self.__plot.invTransform(Qwt.QwtPlot.xBottom, pos.x())
        clickedpointy = self.__plot.invTransform(Qwt.QwtPlot.yLeft, pos.y())

        return Qt.QPoint(int(clickedpointx), int(clickedpointy))

    # def _point_clicked(self, point):
    #     """
    #     This signal is emitted when user click on a plot.
    #     """
    #     self.signal_point_clicked.emit(point)


# def main():
#     app = Qt.QApplication(sys.argv)
#     window = AnnotatorTemp()
#     window.resize(680, 400)
#     window.show()
#     sys.exit(app.exec_())
#
# if __name__ == '__main__':
#     main()
